@@ -20,10 +20,11 @@
 from __future__ import with_statement
 from contextlib import closing
 from functools import wraps, partial
-from httplib import MULTI_STATUS, OK, CONFLICT, NO_CONTENT
+from httplib import MULTI_STATUS, OK, CONFLICT, NO_CONTENT, UNAUTHORIZED
 from StringIO import StringIO
 from xml.etree.ElementTree import ElementTree, Element, SubElement, tostring
 from xml.parsers.expat import ExpatError
+import hashlib
 import httplib
 import sys
 import urllib
@@ -33,7 +34,7 @@ from tinydav.exception import HTTPError, HTTPUserError, HTTPServerError
 
 __author__ = "Manuel Hermann <manuel-hermann@gmx.net>"
 __license__ = "LGPL"
-__version__ = "0.6.2"
+__version__ = "0.6.3"
 
 __all__ = (
     "HTTPError", "HTTPUserError", "HTTPServerError",
@@ -45,7 +46,7 @@ PYTHON2_7 = (sys.version_info >= (2, 7))
 
 # RFC 2518, 9.8 Timeout Request Header
 # The timeout value for TimeType "Second" MUST NOT be greater than 2^32-1.
-MAX_TIMEOUT = 4294967295
+MAX_TIMEOUT = 2**32-1
 
 ACTIVELOCK = "/{DAV:}lockdiscovery/{DAV:}activelock"
 
@@ -94,6 +95,8 @@ class HTTPResponse(int):
         version = "HTTP/%s.%s" % tuple(str(response.version))
         self.statusline = "%s %d %s"\
                         % (version, response.status, response.reason)
+        if self == UNAUTHORIZED:
+            self._setauth()
 
     def __repr__(self):
         """Return representation."""
@@ -103,6 +106,19 @@ class HTTPResponse(int):
         """Return string representation."""
         return self.statusline
 
+    def _setauth(self):
+        value = self.headers.get("www-authenticate", "")
+        auth = util.parse_authenticate(value)
+        self.schema = auth.get("schema")
+        self.realm = auth.get("realm")
+        self.domain = auth.get("domain")
+        self.nonce = auth.get("nonce")
+        self.opaque = auth.get("opaque")
+        stale = auth.get("stale", "false")
+        self.stale = (state.lower() == "true")
+        algorithm = auth.get("algoritm", "MD5")
+        self.algorithm = getattr(hashlib, algorithm.lower())
+
 
 class WebDAVResponse(HTTPResponse):
     """Result from WebDAV request.
@@ -129,7 +145,6 @@ class WebDAVResponse(HTTPResponse):
     multi-status.
 
     """
-
     def __init__(self, response):
         """Initialize the WebDAVResponse.
 
@@ -173,9 +188,18 @@ class WebDAVResponse(HTTPResponse):
             yield self
 
     def _parse_xml_content(self):
-        """Parse the XML content."""
+        """Parse the XML content.
+        
+        If the response content cannot be parsed as XML content, 
+        <root><empty/></root> will be taken as content instead.
+
+        """
         try:
-            self._etree.parse(StringIO(self.content))
+            if self._content is None:
+                parse_me = self.response
+            else:
+                parse_me = StringIO(self.content)
+            self._etree.parse(parse_me)
         except ExpatError, e:
             self.parse_error = e
             # don't fail on further processing
@@ -594,6 +618,7 @@ class HTTPClient(object):
         self.cert_file = None
         self.headers = dict()
         self.cookie = None
+        self._do_digest_auth = False
 
     def _getconnection(self):
         """Return HTTP(S)Connection object depending on set protocol."""
@@ -752,7 +777,7 @@ class HTTPClient(object):
         return self._request("HEAD", uri, None, headers)
 
     def post(self, uri, content="", headers=None, query=None,
-             as_multipart=False, encoding="utf-8"):
+             as_multipart=False, encoding="ascii", with_filenames=False):
         """Make POST request and return HTTPResponse.
 
         uri -- Path to post data to.
@@ -767,7 +792,10 @@ class HTTPClient(object):
                         2-tuple containing the actual value (or file-like 
                         object) and an encoding for this value (or the
                         content-type in case of a file-like object).
-        encoding -- Send multipart content encoding with this encoding.
+        encoding -- Send multipart content encoding with this encoding. Default
+                    is ASCII.
+        with_filenames -- If True, a multipart's files will be sent with the
+                          filename paramenter set. Default is False.
 
         Raise HTTPUserError on 4xx HTTP status codes.
         Raise HTTPServerError on 5xx HTTP status codes.
@@ -776,7 +804,9 @@ class HTTPClient(object):
         (uri, headers) = self._prepare(uri, headers, query)
         if isinstance(content, dict):
             if as_multipart:
-                (multihead, content) = util.make_multipart(content, encoding)
+                (multihead, content) = util.make_multipart(content,
+                                                           encoding,
+                                                           with_filenames)
                 headers.update(multihead)
             else:
                 headers["content-type"] = "application/x-www-form-urlencoded"
